/*
 * RFM12B.h
 *
 *  Created on: 10-06-2013
 *      Author: Piotr Rzeszut
 *  
 * Description: Funkcje do obsugi ukadu RFM12B z poziomu procesora AVR
 */
#ifndef RFM12B_H
#define RFM12B_H

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "RFM12B_reg.h"
#include "crc8.h"

//definicje pinw IO, na ktrych podczona jest magistrala SPI
#define SPI_DDR DDRB
#define SPI_PORT PORTB
#define SPI_PIN PINB

#define SPI_SS PB4
#define SPI_MOSI PB5
#define SPI_MISO PB6
#define SPI_SCK PB7

//pin CS moemy wybra dowolnie
#define CS_DDR DDRB
#define CS_PORT PORTB
#define CS_RFM PB4

//definicja przerwania wyzwalanego stanem niskim
#define RFM_INT_MASK EIMSK
#define RFM_INT_NUM INT0
#define RFM_INT_PORT PORTD
#define RFM_INT_PORT_NUM PD2

//rozmiar bufora musi by naturaln potg liczby 2
#define RFM_BUFFER_SIZE 32

typedef enum{
	RFM_IDLE,//bezczynno
	RFM_RX,//odbir
	RFM_RXC,//odbir zakoczony
	RFM_RXOVF,//przepenienie bufora odbiorczego
	RFM_TX,//nadawanie
	RFM_TXC,//nadawanie zakoczone
	RFM_TXOVF,//nadpisanie bufora nadawczego
} rfm_state_t;

void Rfm_spi_init(void);

uint16_t Rfm_xmit(uint16_t data);

void Rfm_init(void);

uint8_t Rfm_ready_wait(void);

void Rfm_stop(void);

void Rfm_rx_prepare(void);

rfm_state_t Rfm_state_check(void);

void Rfm_rx_get(uint8_t* buffer, uint8_t* num);

void Rfm_tx_start(void);

void Rfm_tx_set(uint8_t* buffer, uint8_t num, uint8_t sync);

void Rfm_tx_frame_prepare(uint8_t* buffer, uint8_t num, uint8_t sync);

uint8_t Rfm_rx_frame_good(uint8_t* buffer, uint8_t* num, uint8_t sync);

#endif //RFM12B_H
