/*
 * main.c
 *
 *  Created on: 19-02-2014
 *      Author: Piotr Rzeszut
 *  
 * Description: Przykad komunikacji z moduem RFM12B
 */
#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>
#include <avr/interrupt.h>

#include "crc8.h"
#include "dallas_one_wire.h"

#include "RFM12B.h"
#include "uart.h"

char bufor_data[32];
char bufor[65];
uint8_t rx_buf[32];

uint8_t buffer[9];
uint8_t ok;
uint16_t measure;
uint8_t subzero;
uint8_t temp_int;
uint16_t temp_fract;

//jeli ponisza linia bdzie zakomentowana to skompilujemy kod dla ukadu SLAVE,
//natomiast jeli bdzie odkomentowana to skompilujemy kod dla ukadu SLAVE
#define MASTER

//adres ukadu master
#define MASTER_ADDR 0xD4
//adresy 2 ukadw slave
#define SLAVE1_ADDR 0xD2
#define SLAVE2_ADDR 0xD0
//ru definiujemy z jakim adresem zostanie skompilowany kod dla ukadu slave
#define CUR_SLAVE_ADDR SLAVE1_ADDR

#ifdef MASTER
//--------------------------------------------------------------------------------------------------------
//Kod dla ukadu MASTER
//--------------------------------------------------------------------------------------------------------
int main(void){

	Rfm_spi_init();//inicjalizacja magistrali SPI

	uart_init(9600);//inicjalizacja USART
	sei();//wczamy przerwania do obsugi uart

	uart_puts("\r\n\r\nRFM12B - MASTER\r\n");//wywietlamy powitanie
	
	Rfm_init();//inicjalizujemy ukad RFM12B
	
	Rfm_xmit(SYNC_PATTERN|MASTER_ADDR);
	//ustawiamy programowalny bajt synchronizacji
	
	uint8_t slave_addr[2]={SLAVE1_ADDR,SLAVE2_ADDR};//definiujemy tablic z adresami 2 urzdze SLAVE
	uint8_t cur_meas=1;//i zmienn informujc o aktualnie wybranym ukadzie
	while(1){
		//za kadym razem zmieniamy aktywny ukad.
		cur_meas=!cur_meas;
		
		//przygotowujemy i wysyamy zapytanie o temperatur do jednego z ukadw
		bufor[0]='T';
		bufor[1]=0;
		//to nowa funkcja, ktra dodaje do danych sum kontroln CRC
		Rfm_tx_frame_prepare((uint8_t*)bufor,strlen(bufor),slave_addr[cur_meas]);
		//wysyanie ju znamy
		Rfm_tx_set((uint8_t*)bufor,strlen(bufor),slave_addr[cur_meas]);
		Rfm_tx_start();
		while(Rfm_state_check()==RFM_TX);
		
		//po wysaniu przygotowujemy si do odczytu
		Rfm_rx_prepare();	
		
		//ustawiamy zmienn z wartoci timeout'u - czasu po ktrym uznamy, e urzdzenie nie odpowiada
		uint8_t timeout=200;
		
		//sprawdzam czy warto jest wiksza od 1 - dziki temu jeli po zakoczeniu ptli warto
		//timeout==1 to znaczy, e ukad nie odpowiedzia
		while(timeout>1){
			//odbieramy dane w znany nam sposb
			uint8_t tmp;
			if(Rfm_state_check()==RFM_RXC){
				
					Rfm_rx_get(rx_buf,&tmp);
					Rfm_rx_prepare();
					//tu kolejna funkcja, ktra sprawdza crc odebranych danych, wraz z adresem docelowym
					ok=Rfm_rx_frame_good(rx_buf,&tmp,MASTER_ADDR);
					//wywietlamy dane
					sprintf(bufor,"DATA(%d): ",cur_meas+1);
					uart_puts(bufor);
					rx_buf[tmp]=0;
					uart_puts((char*)rx_buf);
					//i informacj o zgodnoci sumy kontrolnej (normalnie danych ze zym crc wogle bymy nie obsugiwali)
					if(ok){
						uart_puts(" CRC-OK ");
					}else{
						uart_puts(" CRC-ERR");
					}
					uart_puts("\r\n");
					//ustawiamy timeout na warto 1...
					timeout=1;
			}
			//...dziki czemu tu zostanie on zmieniony na warto 0, ktrej nie osignby gdyby ptla zakoczya si
			//bez wykonania tamtej instrukcji if
			timeout--;
			//czekamy nieco midzy kolejnym sprawdzeniem, czy dane nie nadeszy
			_delay_us(250);
			
		}
		//tu sprawdzamy, czy dane zostay odebrane, czy te nie byo odpowiedzi i wywietlamy wtedy stosowny komunikat
		if(timeout==1){
			sprintf(bufor,"DATA(%d): NOT RESPONDING\r\n",cur_meas+1);
			uart_puts(bufor);	
		}
		//wyczamy odbiornik
		Rfm_stop();
		//i czekamy przed kolejnym zapytaniem o temperatur
		_delay_ms(500);
	}
	return 0;
}

#else
//--------------------------------------------------------------------------------------------------------
//Kod dla ukadu SLAVE
//--------------------------------------------------------------------------------------------------------

volatile uint8_t tick_flag=0;

int main(void){

	Rfm_spi_init();//inicjalizacja magistrali SPI
	
	Rfm_init();//wstpna konfiguracja ukadu RFM12B
	
	Rfm_xmit(SYNC_PATTERN|CUR_SLAVE_ADDR);
	//ustawiamy programowalny bajt synchronizacji
	
	TCCR1B |= (1<<WGM12)|(1<<CS10)|(1<<CS12);//timer 1 - tryb ctc - /1024
	OCR1A = 11800;//warto dla przerwania co okoo 755ms
	TIMSK1 |= (1<<OCIE1A);//zezwolenie na przerwanie od ocr1A
	
	//wywoujemy funkcj pomiaru temperatury
	dallas_reset();//reset magistrali 1-wire
	dallas_write_byte(SKIP_ROM_COMMAND);//pominicie weryfikacji numeru
	dallas_write_byte(CONVERT_T_COMMAND);//zlecamy konwersj temperatury
	
	
	sei();//wczamy gobalny system przerwa.
	
	
	Rfm_rx_prepare();
	while(1){
		uint8_t tmp;
		//odbieramy dane w znany nam sposb
		switch(Rfm_state_check()){
			case RFM_RXC:
				Rfm_rx_get(rx_buf,&tmp);
				//tu sprawdzamy poprawno crc odebranej ramki
				ok=Rfm_rx_frame_good(rx_buf,&tmp,CUR_SLAVE_ADDR);
				if(ok){
					//i jeli prawidowa to czy zawiera zapytanie o temperatur (znak T)
					if(rx_buf[0]=='T'){
						_delay_ms(5);//czekamy nieco - master musi przestawi si na odbir
						//i nadajemy ostatnio przygotowan ramk
						Rfm_tx_set((uint8_t*)bufor,strlen(bufor),MASTER_ADDR);
						Rfm_tx_start();
						while(Rfm_state_check()==RFM_TX);
					}
				}
				Rfm_rx_prepare();
			break;
			case RFM_RXOVF:
				Rfm_rx_prepare();
			break;
			default:break;
		}
		
		//przerwania wyczamy na czas komunikacji 1-wire
		cli();
		if(tick_flag){//wykonamy te instrukcje co 750ms
			tick_flag=0;
			ok=dallas_reset();//reset magistrali 1-wire
			if(!ok){//jeli nie odpowiedzia aden termometr to wywietlamy informacj
				//w kadej sytuacji przygotowujemy stosown informacj
				sprintf(bufor,"T.PRESENT");
				//nastpnie przygotowujemy ramk wraz z sum CRC
				Rfm_tx_frame_prepare((uint8_t*)bufor,strlen(bufor),MASTER_ADDR);
				
				dallas_reset();
				dallas_write_byte(SKIP_ROM_COMMAND);//pominicie weryfikacji numeru
				dallas_write_byte(CONVERT_T_COMMAND);//zlecamy konwersj temperatury
				continue;
			}

			dallas_reset();//reset magistrali 1-wire
			dallas_write_byte(SKIP_ROM_COMMAND);//pominicie weryfikacji numeru
			dallas_write_byte(READ_SCRATCHPAD_COMMAND);//zlecamy odczyt danych
			dallas_read_buffer(buffer,9);//odczytujemy dane z termometru
			if(buffer[8]!=crc8(buffer,8)){//sprawdzamy sum kontroln odczytu
				sprintf(bufor,"T.CRC8.ERR");
			
				Rfm_tx_frame_prepare((uint8_t*)bufor,strlen(bufor),MASTER_ADDR);
				
				dallas_reset();
				dallas_write_byte(SKIP_ROM_COMMAND);//pominicie weryfikacji numeru
				dallas_write_byte(CONVERT_T_COMMAND);//zlecamy konwersj temperatury
				continue;
			}
			measure=(uint16_t)buffer[0]+(((uint16_t)buffer[1])<<8);//czymy 2 bajty danych o temperaturze
			if(measure&0x8000){//jeli wynik jest ujemny to zapisujemy informacj o znaku i konwertujemy liczb kodu U2 na dodatni
				subzero=1;
				measure ^= 0xFFFF;
				measure += 1;
				}else{
				subzero=0;
			}
			//rozdzielamy liczb na cz cakowit i uamkow
			temp_int=measure>>4;
			temp_fract=(measure&0x000F)*625;
			sprintf(bufor,"%c%03d.%04d",subzero?'-':' ',temp_int,temp_fract);
		
			Rfm_tx_frame_prepare((uint8_t*)bufor,strlen(bufor),MASTER_ADDR);
			
			dallas_reset();
			dallas_write_byte(SKIP_ROM_COMMAND);//pominicie weryfikacji numeru
			dallas_write_byte(CONVERT_T_COMMAND);//zlecamy konwersj temperatury
		}
		sei();
	}

	return 0;
}

ISR(TIMER1_COMPA_vect){//przerwanie co ~750ms
	tick_flag=1;//ustawia flag odmierzenia tego czasu
}

#endif
